"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var xml_js_1 = require("xml-js");
var utils_1 = require("./utils");
var actions = require("./actions");
function stateNodeToSCXML(stateNode) {
    var parallel = stateNode.parallel;
    var scxmlElement = {
        type: 'element',
        name: parallel ? 'parallel' : 'state',
        attributes: {
            id: stateNode.id
        },
        elements: [
            !parallel && stateNode.initial
                ? {
                    type: 'element',
                    name: 'initial',
                    elements: [
                        {
                            type: 'element',
                            name: 'transition',
                            attributes: {
                                target: stateNode.states[stateNode.initial].id
                            }
                        }
                    ]
                }
                : undefined,
            stateNode.onEntry && {
                type: 'element',
                name: 'onentry',
                elements: stateNode.onEntry.map(function (event) {
                    return {
                        type: 'element',
                        name: 'send',
                        attributes: {
                            event: utils_1.getEventType(event)
                        }
                    };
                })
            },
            stateNode.onExit && {
                type: 'element',
                name: 'onexit',
                elements: stateNode.onExit.map(function (event) {
                    return {
                        type: 'element',
                        name: 'send',
                        attributes: {
                            event: utils_1.getEventType(event)
                        }
                    };
                })
            }
        ].concat(Object.keys(stateNode.states).map(function (stateKey) {
            var subStateNode = stateNode.states[stateKey];
            return stateNodeToSCXML(subStateNode);
        }), (stateNode.on
            ? Object.keys(stateNode.on)
                .map(function (event) {
                var transition = stateNode.on[event];
                if (!transition) {
                    return [];
                }
                if (Array.isArray(transition)) {
                    return transition.map(function (targetTransition) {
                        return {
                            type: 'element',
                            name: 'transition',
                            attributes: __assign({}, event ? { event: event } : undefined, { target: stateNode.parent.getState(targetTransition.target).id }, targetTransition.cond
                                ? { cond: targetTransition.cond.toString() }
                                : undefined),
                            elements: targetTransition.actions
                                ? targetTransition.actions.map(function (action) { return ({
                                    type: 'element',
                                    name: 'send',
                                    attributes: {
                                        event: utils_1.getEventType(action)
                                    }
                                }); })
                                : undefined
                        };
                    });
                }
                if (typeof transition === 'string') {
                    return [
                        {
                            type: 'element',
                            name: 'transition',
                            attributes: __assign({}, event ? { event: event } : undefined, { target: stateNode.parent.getState(stateNode.on[event]).id })
                        }
                    ];
                }
                return Object.keys(transition).map(function (target) {
                    var targetTransition = transition[target];
                    return {
                        type: 'element',
                        name: 'transition',
                        attributes: __assign({}, event ? { event: event } : undefined, { target: stateNode.parent.getState(target).id }, targetTransition.cond
                            ? { cond: targetTransition.cond.toString() }
                            : undefined),
                        elements: targetTransition.actions
                            ? targetTransition.actions.map(function (action) { return ({
                                type: 'element',
                                name: 'send',
                                attributes: {
                                    event: utils_1.getEventType(action)
                                }
                            }); })
                            : undefined
                    };
                });
            })
                .reduce(function (a, b) { return a.concat(b); })
            : [])).filter(Boolean)
    };
    return scxmlElement;
}
function fromMachine(machine) {
    var scxmlDocument = {
        declaration: { attributes: { version: '1.0', encoding: 'utf-8' } },
        elements: [
            { type: 'instruction', name: 'access-control', instruction: 'allow="*"' },
            {
                type: 'element',
                name: 'scxml',
                attributes: {
                    version: '1.0',
                    initial: machine.id
                }
            },
            stateNodeToSCXML(machine)
        ]
    };
    return xml_js_1.js2xml(scxmlDocument, { spaces: 2 });
}
exports.fromMachine = fromMachine;
function indexedRecord(items, identifier) {
    var record = {};
    var identifierFn = typeof identifier === 'string' ? function (item) { return item[identifier]; } : identifier;
    items.forEach(function (item) {
        record[identifierFn(item)] = item;
    });
    return record;
}
function executableContent(elements) {
    var transition = {
        actions: []
    };
    elements.forEach(function (element) {
        switch (element.name) {
            case 'raise':
                transition.actions.push(actions.raise(element.attributes.event));
            default:
                return;
        }
    });
    return transition;
}
function toConfig(nodeJson) {
    var initial = nodeJson.attributes.initial;
    var states;
    var on;
    if (nodeJson.elements) {
        var stateElements = nodeJson.elements.filter(function (element) { return element.name === 'state'; });
        var transitionElements = nodeJson.elements.filter(function (element) { return element.name === 'transition'; });
        var onEntryElement = nodeJson.elements.find(function (element) { return element.name === 'onentry'; });
        var onExitElement = nodeJson.elements.find(function (element) { return element.name === 'onexit'; });
        states = indexedRecord(stateElements, function (item) { return "" + item.attributes.id; });
        on = utils_1.mapValues(indexedRecord(transitionElements, function (item) { return item.attributes.event || ''; }), function (value) {
            return [
                __assign({ target: value.attributes.target }, value.elements ? executableContent(value.elements) : undefined)
            ];
        });
        var onEntry = onEntryElement
            ? onEntryElement.elements.map(function (element) {
                switch (element.name) {
                    case 'raise':
                        return actions.raise(element.attributes.event);
                    default:
                        return 'not-implemented';
                }
            })
            : undefined;
        var onExit = onExitElement
            ? onExitElement.elements.map(function (element) {
                switch (element.name) {
                    case 'raise':
                        return actions.raise(element.attributes.event);
                    default:
                        return 'not-implemented';
                }
            })
            : undefined;
        return __assign({}, initial ? { initial: initial } : undefined, stateElements.length
            ? { states: utils_1.mapValues(states, toConfig) }
            : undefined, transitionElements.length ? { on: on } : undefined, onEntry ? { onEntry: onEntry } : undefined, onExit ? { onExit: onExit } : undefined);
    }
    return {};
}
function toMachine(xml) {
    var json = xml_js_1.xml2js(xml);
    var machineElement = json.elements.filter(function (element) { return element.name === 'scxml'; })[0];
    return toConfig(machineElement);
}
exports.toMachine = toMachine;
